﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace IPv6_Multicast {
    class Program {
        static void Main(string[] args) {
            var endpoint = new IPEndPoint(IPAddress.Parse("ff0e::1234"), 8080);

            var socket = new Socket(AddressFamily.InterNetworkV6, SocketType.Dgram, ProtocolType.Udp);
            socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
            socket.Bind(new IPEndPoint(IPAddress.IPv6Any, endpoint.Port));
            socket.SetSocketOption(SocketOptionLevel.IPv6, SocketOptionName.AddMembership, new IPv6MulticastOption(endpoint.Address));

            Console.WriteLine("Waiting for a key on " + endpoint.ToString() + "...");
            Console.WriteLine();
            while (true) {
                if (Console.KeyAvailable) {
                    var key = Console.ReadKey(true);
                    if (key.Key == ConsoleKey.Escape) { break; }
                    var buffer = Encoding.UTF8.GetBytes(key.Key.ToString());
                    socket.SendTo(buffer, endpoint);
                } else if (socket.Available > 0) {
                    var remoteEP = (EndPoint)(new IPEndPoint(IPAddress.IPv6Any, 0));
                    byte[] buffer = new Byte[4096];
                    var len = socket.ReceiveFrom(buffer, ref remoteEP);
                    var text = Encoding.UTF8.GetString(buffer, 0, len);
                    Console.WriteLine(text + " " + remoteEP.ToString());
                }
            }

            socket.SetSocketOption(SocketOptionLevel.IPv6, SocketOptionName.DropMembership, new IPv6MulticastOption(endpoint.Address));
        }
    }
}
